import {
  setPlayState,
  selectSeconds,
  setPlaySeconds,
} from "../player/PlayerSlice";

import {
  selectConnected,
  setConnectionStatus,
} from "../connectionstatus/ConnectionStatusSlice";

import { all, put, select, takeEvery } from "redux-saga/effects";

export const requestPlay = {
  type: "REDUX_WEBSOCKET::SEND",
  payload: { command: "play", target: "server" },
};

export const requestPause = {
  type: "REDUX_WEBSOCKET::SEND",
  payload: { command: "pause", target: "server" },
};

export const requestSyncAll = {
  type: "REDUX_WEBSOCKET::SEND",
  payload: { command: "sync", target: "server" },
};

export const requestSyncToMe = (seconds: number) => {
  return {
    type: "REDUX_WEBSOCKET::SEND",
    payload: {
      command: "sync_to_time",
      playback_time: seconds,
      target: "server",
    },
  };
};

export const sendPlayerTime = (seconds: number) => {
  return {
    type: "REDUX_WEBSOCKET::SEND",
    payload: { playback_time: seconds },
  };
};

export type backendMessageFormat = {
  type: "REDUX_WEBSOCKET::MESSAGE";
  meta: {
    timestamp: Date;
  };
  payload: {
    message: string;
    origin: string;
  };
};

export type connectionStatusFormat = {
  type: "REDUX_WEBSOCKET::OPEN" | "REDUX_WEBSOCKET::CLOSED";
  meta: {
    timestamp: Date;
  };
};

// worker Saga: will be fired on USER_FETCH_REQUESTED actions
function* fetchUser(action: any) {
  try {
    //const user = yield call(Api.fetchUser, action.payload.userId);
    console.log(action);
    let backendmessage = action as backendMessageFormat;

    let message = JSON.parse(backendmessage.payload.message) as {
      command: string;
      data?: number;
    };

    switch (message.command) {
      case "play": {
        yield put(setPlayState(true));
        break;
      }
      case "pause": {
        yield put(setPlayState(false));
        break;
      }
      case "get-playback-time": {
        let playtime: number = yield select(selectSeconds);
        console.log(playtime);
        yield put(sendPlayerTime(playtime));
        break;
      }
      case "set-playback-time": {
        if (message.data === undefined) return;
        yield put(setPlaySeconds(message.data));
      }
    }
    //yield put({type: "USER_FETCH_SUCCEEDED", user: user});
  } catch (e) {
    //yield put({type: "USER_FETCH_FAILED", message: e.message});
  }
}

function* messageSaga() {
  yield takeEvery("REDUX_WEBSOCKET::MESSAGE", fetchUser);
}

function* handleConnectionStatus(action: any) {
  let connectionMessage = action as connectionStatusFormat;

  switch (connectionMessage.type) {
    case "REDUX_WEBSOCKET::OPEN": {
      yield put(setConnectionStatus(true));
      break;
    }
    case "REDUX_WEBSOCKET::CLOSED": {
      let isConnect: boolean | null = yield select(selectConnected);
      if (isConnect !== null) {
        yield put(setConnectionStatus(false));
      }
      break;
    }
  }
  yield;
}

function* connectionStatusSaga() {
  // "REDUX_WEBSOCKET::BROKEN" is not neccessary, because it comes after CLOSED
  // Maybe add ERROR
  yield takeEvery(
    ["REDUX_WEBSOCKET::OPEN", "REDUX_WEBSOCKET::CLOSED"],
    handleConnectionStatus
  );
}

export default function* rootSaga() {
  yield all([connectionStatusSaga(), messageSaga()]);
}
