import React from "react";
import { connect } from "react-redux";
import PlayerControlButton from "./PlayerControlButton";

import { requestSyncToMe } from "../backendconnection/communication";
import { RootState } from "../store/store";
import { selectSeconds } from "./PlayerSlice";
import Replay10 from "@material-ui/icons/Replay10";

interface StateProps {
  seconds: number;
}

interface DispatchProps {
  sync: (seconds: number) => void;
}

export type Props = StateProps & DispatchProps;

class InstantReplayButton extends React.Component<Props> {
  constructor(props: Props) {
    super(props);
    this.handleSync = this.handleSync.bind(this);
  }

  handleSync(event: React.MouseEvent<HTMLButtonElement, MouseEvent>): void {
    this.props.sync(this.props.seconds);
  }

  render() {
    return (
      <PlayerControlButton
        icon={<Replay10 />}
        content="Instant Replay!"
        onClick={this.handleSync}
      ></PlayerControlButton>
    );
  }
}

const mapStateToProps = (state: RootState): StateProps => ({
  seconds: selectSeconds(state),
});

const mapDispatchToProps = (dispatch: Function): DispatchProps => ({
  sync(seconds: number): void {
    dispatch(requestSyncToMe(seconds - 10));
  },
});

export default connect(
  mapStateToProps,
  mapDispatchToProps
)(InstantReplayButton);
