import { createSlice, PayloadAction } from "@reduxjs/toolkit";
import { RootState } from "../store/store";

export interface PlayerState {
  playing: boolean;
  seconds: number;
}

export const initialState: PlayerState = {
  playing: false,
  seconds: 0,
};

export type PlayerPlayingAction = PayloadAction<boolean>;
export type PlayerSecondsAction = PayloadAction<number>;

export const PlayerStateSlice = createSlice({
  name: "player",
  initialState,
  reducers: {
    setPlayState: (state: PlayerState, action: PlayerPlayingAction): void => {
      state.playing = action.payload;
    },
    setPlaySeconds: (state: PlayerState, action: PlayerSecondsAction): void => {
      state.seconds = action.payload;
    },
  },
});

export const { setPlayState, setPlaySeconds } = PlayerStateSlice.actions;

export const selectPlaying = (state: RootState) => state.player.playing;
export const selectSeconds = (state: RootState) => state.player.seconds;

export default PlayerStateSlice.reducer;
