import { configureStore, getDefaultMiddleware } from "@reduxjs/toolkit";
import reduxWebsocket from "@giantmachines/redux-websocket";

import PlayerStateReducer from "../player/PlayerSlice";
import ConnectionStatusStateReducer from "../connectionstatus/ConnectionStatusSlice";
import UrlReducer from "../url/UrlSlice";

import rootSaga from "../backendconnection/communication";

import createSagaMiddleware from "redux-saga";

import { connect } from "@giantmachines/redux-websocket";

// Create the websocket instance.
const reduxWebsocketMiddleware = reduxWebsocket({
  reconnectInterval: 1000,
  reconnectOnClose: true,
  dateSerializer: (date: Date) => "haha",
});
// See https://github.com/giantmachines/redux-websocket
// and https://www.npmjs.com/package/@giantmachines/redux-websocket

// create the saga middleware
const sagaMiddleware = createSagaMiddleware();

const initialState = {};

// Initialize the store
export const store = configureStore({
  preloadedState: initialState,
  reducer: {
    url: UrlReducer,
    player: PlayerStateReducer,
    connectionStatus: ConnectionStatusStateReducer,
  },
  middleware: [
    ...getDefaultMiddleware({
      thunk: false,
      serializableCheck: {
        // Ignore these action types
        ignoredActions: [
          "REDUX_WEBSOCKET::CONNECT",
          "REDUX_WEBSOCKET::CLOSED",
          "REDUX_WEBSOCKET::ERROR",
          "REDUX_WEBSOCKET::RECONNECT_ATTEMPT",
          "REDUX_WEBSOCKET::RECONNECTED",
          "REDUX_WEBSOCKET::BEGIN_RECONNECT",
          "REDUX_WEBSOCKET::BROKEN",
          "REDUX_WEBSOCKET::OPEN",
          "REDUX_WEBSOCKET::MESSAGE",
          "REDUX_WEBSOCKET::SEND",
        ],
      },
    }),
    sagaMiddleware,
    reduxWebsocketMiddleware,
  ],
});

const domain = window.location.hostname;
const group = window.location.pathname;

let ws_url = `wss://${domain}/api/ws/${group}`

// It's localhost, no SSL needed / possible.
if(domain === 'localhost'){
  ws_url = `ws://${domain}:8432/${group}`
}

store.dispatch(connect(ws_url));
sagaMiddleware.run(rootSaga);

export type RootState = ReturnType<typeof store.getState>;
